@@ -32,9 +32,9 @@
  */
 
 #include "squid.h"
-#include "acl/Checklist.h"
 #include "acl/MyPortName.h"
 #include "acl/StringData.h"
+#include "acl/Checklist.h"
 #include "anyp/PortCfg.h"
 #include "HttpRequest.h"
 
@@ -42,7 +42,7 @@
 #include "client_side.h"
 
 int
-ACLMyPortNameStrategy::match(ACLData<MatchType> * &data, ACLFilledChecklist *checklist, ACLFlags &)
+ACLMyPortNameStrategy::match(ACLData<MatchType> * &data, ACLFilledChecklist *checklist)
 {
     if (checklist->conn() != NULL && checklist->conn()->port != NULL)
         return data->match(checklist->conn()->port->name);
