@@ -53,9 +53,9 @@
 #include "ExternalACL.h"
 #include "fd.h"
 #include "format/Token.h"
+#include "forward.h"
 #include "fs/Module.h"
 #include "fqdncache.h"
-#include "FwdState.h"
 #include "globals.h"
 #include "htcp.h"
 #include "HttpHeader.h"
@@ -120,10 +120,10 @@
 #include "LoadableModules.h"
 #endif
 #if USE_SSL_CRTD
+#include "ssl/helper.h"
 #include "ssl/certificate_db.h"
 #endif
 #if USE_SSL
-#include "ssl/helper.h"
 #include "ssl/context_storage.h"
 #endif
 #if ICAP_CLIENT
@@ -153,6 +153,7 @@
 #endif
 
 #if USE_WIN32_SERVICE
+#include "squid_windows.h"
 #include <process.h>
 
 static int opt_install_service = FALSE;
@@ -161,6 +162,11 @@ static int opt_signal_service = FALSE;
 static int opt_command_line = FALSE;
 void WIN32_svcstatusupdate(DWORD, DWORD);
 void WINAPI WIN32_svcHandler(DWORD);
+
+#endif
+
+#if !defined(SQUID_BUILD_INFO)
+#define SQUID_BUILD_INFO ""
 #endif
 
 static char *opt_syslog_facility = NULL;
@@ -296,11 +302,7 @@ usage(void)
 #if USE_WIN32_SERVICE
             "       -i        Installs as a Windows Service (see -n option).\n"
 #endif
-            "       -k reconfigure|rotate|shutdown|"
-#ifdef SIGTTIN
-            "restart|"
-#endif
-            "interrupt|kill|debug|check|parse\n"
+            "       -k reconfigure|rotate|shutdown|interrupt|kill|debug|check|parse\n"
             "                 Parse configuration file, then send signal to \n"
             "                 running copy (except -k parse) and exit.\n"
 #if USE_WIN32_SERVICE
@@ -505,6 +507,12 @@ mainParseOptions(int argc, char *argv[])
                 fatal("Need to add -DMALLOC_DBG when compiling to use -mX option");
 #endif
 
+            } else {
+#if XMALLOC_TRACE
+                xmalloc_trace = !xmalloc_trace;
+#else
+                fatal("Need to configure --enable-xmalloc-debug-trace to use -m option");
+#endif
             }
             break;
 
@@ -698,6 +706,7 @@ serverConnectionsOpen(void)
         snmpOpenPorts();
 #endif
 
+        clientdbInit();
         icmpEngine.Open();
         netdbInit();
         asnInit();
@@ -760,8 +769,6 @@ mainReconfigureStart(void)
     Ssl::Helper::GetInstance()->Shutdown();
 #endif
 #if USE_SSL
-    if (Ssl::CertValidationHelper::GetInstance())
-        Ssl::CertValidationHelper::GetInstance()->Shutdown();
     Ssl::TheGlobalContextStorage.reconfigureStart();
 #endif
     redirectShutdown();
@@ -844,10 +851,6 @@ mainReconfigureFinish(void *)
 #if USE_SSL_CRTD
     Ssl::Helper::GetInstance()->Init();
 #endif
-#if USE_SSL
-    if (Ssl::CertValidationHelper::GetInstance())
-        Ssl::CertValidationHelper::GetInstance()->Init();
-#endif
 
     redirectInit();
 #if USE_AUTH
@@ -944,42 +947,23 @@ setEffectiveUser(void)
     }
 }
 
-/// changes working directory, providing error reporting
-static bool
-mainChangeDir(const char *dir)
-{
-    if (chdir(dir) == 0)
-        return true;
-
-    debugs(50, DBG_CRITICAL, "cannot change current directory to " << dir <<
-           ": " << xstrerror());
-    return false;
-}
-
-/// set the working directory.
 static void
 mainSetCwd(void)
 {
-    static bool chrooted = false;
-    if (Config.chroot_dir && !chrooted) {
-        chrooted = true;
+    char pathbuf[MAXPATHLEN];
 
-        if (chroot(Config.chroot_dir) != 0)
-            fatalf("chroot to %s failed: %s", Config.chroot_dir, xstrerror());
-
-        if (!mainChangeDir("/"))
-            fatalf("chdir to / after chroot to %s failed", Config.chroot_dir);
-    }
-
-    if (Config.coredump_dir && strcmp("none", Config.coredump_dir) != 0) {
-        if (mainChangeDir(Config.coredump_dir)) {
+    if (Config.coredump_dir) {
+        if (0 == strcmp("none", Config.coredump_dir)) {
+            (void) 0;
+        } else if (chdir(Config.coredump_dir) == 0) {
             debugs(0, DBG_IMPORTANT, "Set Current Directory to " << Config.coredump_dir);
             return;
+        } else {
+            debugs(50, DBG_CRITICAL, "chdir: " << Config.coredump_dir << ": " << xstrerror());
         }
     }
 
     /* If we don't have coredump_dir or couldn't cd there, report current dir */
-    char pathbuf[MAXPATHLEN];
     if (getcwd(pathbuf, MAXPATHLEN)) {
         debugs(0, DBG_IMPORTANT, "Current Directory is " << pathbuf);
     } else {
@@ -991,7 +975,10 @@ static void
 mainInitialize(void)
 {
     /* chroot if configured to run inside chroot */
-    mainSetCwd();
+
+    if (Config.chroot_dir && (chroot(Config.chroot_dir) != 0 || chdir("/") != 0)) {
+        fatal("failed to chroot");
+    }
 
     if (opt_catch_signals) {
         squid_signal(SIGSEGV, death, SA_NODEFER | SA_RESETHAND);
@@ -1061,11 +1048,6 @@ mainInitialize(void)
     Ssl::Helper::GetInstance()->Init();
 #endif
 
-#if USE_SSL
-    if (Ssl::CertValidationHelper::GetInstance())
-        Ssl::CertValidationHelper::GetInstance()->Init();
-#endif
-
     redirectInit();
 #if USE_AUTH
     authenticateInit(&Auth::TheConfig);
@@ -1300,6 +1282,10 @@ SquidMain(int argc, char **argv)
 {
     ConfigureCurrentKid(argv[0]);
 
+#if HAVE_SBRK
+    sbrk_start = sbrk(0);
+#endif
+
     Debug::parseOptions(NULL);
     debug_log = stderr;
 
@@ -1430,8 +1416,11 @@ SquidMain(int argc, char **argv)
     /* send signal to running copy and exit */
     if (opt_send_signal != -1) {
         /* chroot if configured to run inside chroot */
-        mainSetCwd();
+
         if (Config.chroot_dir) {
+            if (chroot(Config.chroot_dir))
+                fatal("failed to chroot");
+
             no_suid();
         } else {
             leave_suid();
@@ -1453,7 +1442,10 @@ SquidMain(int argc, char **argv)
 
     if (opt_create_swap_dirs) {
         /* chroot if configured to run inside chroot */
-        mainSetCwd();
+
+        if (Config.chroot_dir && chroot(Config.chroot_dir)) {
+            fatal("failed to chroot");
+        }
 
         setEffectiveUser();
         debugs(0, DBG_CRITICAL, "Creating missing swap directories");
@@ -1848,10 +1840,6 @@ SquidShutdown()
     dnsShutdown();
 #if USE_SSL_CRTD
     Ssl::Helper::GetInstance()->Shutdown();
-#endif
-#if USE_SSL
-    if (Ssl::CertValidationHelper::GetInstance())
-        Ssl::CertValidationHelper::GetInstance()->Shutdown();
 #endif
     redirectShutdown();
     externalAclShutdown();
@@ -1920,6 +1908,15 @@ SquidShutdown()
     eventFreeMemory();
     mimeFreeMemory();
     errorClean();
+#endif
+#if !XMALLOC_TRACE
+
+    if (opt_no_daemon) {
+        file_close(0);
+        file_close(1);
+        file_close(2);
+    }
+
 #endif
     // clear StoreController
     Store::Root(NULL);
@@ -1930,6 +1927,13 @@ SquidShutdown()
 
     memClean();
 
+#if XMALLOC_TRACE
+
+    xmalloc_find_leaks();
+
+    debugs(1, DBG_CRITICAL, "Memory used after shutdown: " << xmalloc_total);
+
+#endif
 #if MEM_GEN_TRACE
 
     log_trace_done();
